#ifndef __font_bdf_h
#define __font_bdf_h

#include <cstdio>
#include <new>

typedef struct {
	int w;	
	int h;
	int x;
	int y;
	int d;	
	unsigned char *bmp;	/* 長さは size * (w + 7) / 8 [byte], size == height*/
} bdf_glyph;

/*!	bdfファイルを読み込む

	クラス内部ではJISコードを仕様

	例外
	#include<new> bad_alloc

*/
class font_bdf{
public:
	font_bdf(void);
	~font_bdf(void);

	// FIXME returnが不正
	int open(char *file_name);
		/*!	return
			0:成功
			1:ファイルのオープンに失敗
			2:ファイルが不正
		*/

	void close(void);

	int get_index(char high, char low);
		/* JIS Code
		0xff は accii 
		index = 0xff + high * (0x7e - 0x21 + 1) + low
		9092 = 0xff + (0x7e - 0x21 + 1) * (0x7e - 0x21 + 1);
		*/

	bdf_glyph *glyph[9092];

private:

	/*! return 文字列を描画したときのピクセル長
			0:失敗
	*/
	int str_pixels(char *text);

	/*! return height[pixel] */
	int font_height(void);

	/*! モノクロBMP, lineは一行の長さ[byte] */
	unsigned char *get_pixel_data(char *text, int *line);


	/*! ファイル中で最初に*strが出てくる行から、num_v個の値を読み取る
		str == 0 なら行頭から値として読み取る
		return 0:成功
		2:指定された数の値が存在しない
		3:ファイルの終端に到達

		example
		BBX 12 3 -1 0
		get_value(fp, "BBX", v, 4)
		v[0] = 12, v[1] = 3, ...となる
	*/
	int get_value(FILE *fp, const char *str, long int *v, int num_v, int radix);

};

#endif

