
#ifndef __opengl_texture_h__
#define __opengl_texture_h__

#include <GL/gl.h>
#include "image.h"

#include <iostream>
using namespace std;

/*	大きなテクスチャを一枚確保して、使用するimageをテキスチャに貼り付けて使用する。
 *	textureのサイズはどんどん大きくなるので注意
 */
class opengl_texture{
public:
	opengl_texture(void);
	~opengl_texture(void);

	int make_texture(image *img);
		// return 0 : success
		// return -1: error

	// FIXME int set_texture_size(int size);

	GLuint texture_number;
	image *texture_image;

	unsigned char *texels;
	unsigned int texels_size;

	unsigned int width;
	unsigned int height;
	GLenum format;

	// FIXME internal_format
private:
};

opengl_texture::opengl_texture(void){
	texture_number = 0;
	texture_image = 0;

	texels = 0;
	texels_size = 1;
}

opengl_texture::~opengl_texture(void){
	// FIXME テクスチャオブジェクトの解放

	if(!texels) free(texels);
}

int opengl_texture::make_texture(image *img){
	if(img == 0) return -1;
	texture_image = img;

// opengl
	//ピクセル格納モードの制御
	// ASLIGNMET : アラインメント : バイトの並び
	// この場合は一列(8n + m)ビットの並びを強制的に一列(n + 1)バイトで表現する
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	// テクスチャオブジェクト
	glGenTextures(1, &texture_number); //名前を付ける
	glBindTexture(GL_TEXTURE_2D, texture_number); //作成

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

// 各種パラメータを取得
	width = 1;
	while(width < texture_image->width){
		width = width << 1;
	}
	height = 1;
	while(height < texture_image->height){
		height = height << 1;
	}

	switch(texture_image->get_color_type()){
		case image::RGB:
			format = GL_RGB;
			break;
		case image::RGB_ALPHA:
			format = GL_RGBA;
			break;
		case image::GRAY:
		case image::GRAY_ALPHA:
		case image::MASK:
		case image::PALETTE:
			// FIXME
			// format = GL_COLOR_INDEX;
		default:
			cout << "error opengl_texture.h: bad color type image::get_color_type()" << endl;
			/*	FIXME
				format = GL_DEPTH_COMPONENT;
				format = GL_LUMINANCE;
				format = GL_LUMINANCE_ALPHA;
				format = GL_RED;
				format = GL_GREEN;
				format = GL_BLUE;
				format = GL_ALPHA;
			*/
	}

// GLvoid *texels の確保
	if(texels_size < width * height * texture_image->bit_per_pixel){
		while(texels_size < width * height * texture_image->bit_per_pixel){
			texels_size = texels_size << 1;
		}
		if(!texels){
			texels = (unsigned char*)realloc(texels, texels_size);
		}else{
			texels = (unsigned char*)malloc(texels_size);
		}
	}

// 作成
	// テクスチャの大きさ、種類、位置などを指定する
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,
			width, height, 0, format, GL_UNSIGNED_BYTE, texels);

// 置き換え
	// FIXME if(format == GL_COLOR_INDEX)
	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0,
			texture_image->width, texture_image->height,
			GL_RGB, GL_UNSIGNED_BYTE, texture_image->pixels);
	return 0;
}

#endif
