
#ifndef __libharu_vector3__
#define __libharu_vector3__

// TYPE型 3次元ベクトル
// 内部では直交座標を使用している。
// 極座標演算を頻繁に使用するなら、専用のクラスを使ったほうが速い
template <class TYPE> class vector3;
typedef vector3<double> vector3d;

/* FIXME
	インターフェイスを分離して
	直交座標と極座標の二つのクラスを作る
*/

// FIXME Doxygen対応

// FIXME 名前を変更
// 3 demension vector on rectangular coordinate system

// 他必要な関数を随時追加
// テンプレートを使ったメンバ関数の定義は、ヘッダファイル側に記述する必要がある

/* FIXME
	整数型
	typedef vector3<int> vector3i;
	型キャストが必要か
	x = (TYPE)0.0;
	使えない関数も多い
*/

#include <math.h>
// FIXME M_PIなどの定数の扱い

// for Debug
#include <iostream>
#include <iomanip>

#if 0
// FIXME 要テスト
typedef vector3<float> vector3f;
typedef vector3<int> vector3i;
typedef vector3<char> vector3c;
#endif

template <class TYPE>
class vector3{
public:
	// Constructors
#if 0
	vector3(void):x(0.0), y(0.0), z(0.0){};
	vector3(TYPE X, TYPE Y, TYPE Z):x(X), y(Y), z(Z){};
	vector3(TYPE *src):x(src[0]), y(src[1]), z(src[2]){};
#endif
	vector3(void);
	vector3(TYPE X, TYPE Y, TYPE Z);
	vector3(TYPE *src);
	
	/* FIXME
		返り値
		vector3& return *this
		void return
		の違いは?
	*/
	void set(TYPE X, TYPE Y, TYPE Z);
	void set(TYPE *src);

	// compare
	bool operator== (const vector3 &dst) const;
	bool operator!= (const vector3 &dst) const;

	// calculate
	// FIXME 代入演算子
	vector3 operator+ () const;
	vector3 operator- () const;
	vector3 operator+ (const vector3 &add) const;
	vector3 operator- (const vector3 &sub) const;
	vector3 operator* (TYPE mul) const;
	vector3 & operator+= (const vector3 &add);
	vector3 & operator-= (const vector3 &sub);
	vector3 & operator*= (TYPE mul);

	// 内積
	TYPE dot_product(const vector3 &dotVector) const;
	// 外積
	vector3 cross_product(const vector3 &crossVector) const;
	// 正規化(r=1)
	vector3 & normalize(void);
/*
	vector3 & inverse ()
	vector3 & abs ()
	TYPE 	dotProduct (const vector3 &dotVector) const
	vector3 	crossProduct (const vector3 &crossVector) const
	TYPE 	getLength () const
	vector3 & 	setLength (TYPE length)
	TYPE 	getSquaredLength () const
	vector3 & 	normalize ()
	bool isZero () const
	bool isUnit () const
	TYPE maximumValue () const
	TYPE minimumValue () const
	bool epsilonEquals (const vector3 &target, TYPE epsilon) const
	bool notEpsilonEquals (const vector3 &target, TYPE epsilon) const
*/

	// Access methods
	TYPE get_x(void) const;
	TYPE get_y(void) const;
	TYPE get_z(void) const;

// 球極座標 Spherical Polar Coordinates
	// 遅いので注意
	// 使わないほうがいい
	// (x, y, z) = (r*sin(theta)*cos(phi),
	//				r*sin(theta)*sin(phi),
	//				r*cos(theta))
	// 角度は(PI < theta < -PI)[radian]で表す
	TYPE get_r(void);
	TYPE get_theta(void);
	TYPE get_phi(void);
	void set_polar_coordinate(double r, double theta, double phi);
	// void set_polar_coodinate(double r, double theta, double phi);

	// 変形
	// FIXME function name
	// ベクトルの長さをd増やす,回転
	void transform(double d_r, double d_theta, double d_phi);
	
	// for Debug
	void output(void);

	// FIXME コピーコンストラクタ、代入演算子の扱い
	TYPE x;
	TYPE y;
	TYPE z;
private: // FIXME
};

// FIXME template中のconstructorの定義はこれでいいのか?
template <class TYPE>
vector3<TYPE>::vector3(void){
	x = (TYPE)0.0;
	y = (TYPE)0.0;
	z = (TYPE)0.0;
}

template <class TYPE>
vector3<TYPE>::vector3(TYPE X, TYPE Y, TYPE Z) : x(X), y(Y), z(Z){
}

template <class TYPE>
vector3<TYPE>::vector3(TYPE *src) : x(src[0]), y(src[1]), z(src[2]){
}

template <class TYPE>
bool vector3<TYPE>::operator== (const vector3<TYPE> &dst) const{
	return ((x == dst.x) && (y == dst.y) && (z == dst.z));
}

template <class TYPE>
bool vector3<TYPE>::operator!= (const vector3<TYPE> &dst) const{
	return ((x != dst.x) || (y != dst.y) || (z == dst.z));
}

template <class TYPE>
void vector3<TYPE>::set(TYPE X, TYPE Y, TYPE Z){
	x = X;
	y = Y;
	z = Z;
}


template <class TYPE>
void vector3<TYPE>::set(TYPE *src){
	x = src[0];
	y = src[1];
	z = src[2];
}


template <class TYPE>
vector3<TYPE> vector3<TYPE>::operator+ () const{
	return *this;
}

template <class TYPE>
vector3<TYPE> vector3<TYPE>::operator- () const{
	return vector3<TYPE>(-x, -y, -z);
}

template <class TYPE>
vector3<TYPE> vector3<TYPE>::operator+ (const vector3<TYPE> &add) const{
	return vector3<TYPE>(x + add.x, y + add.y, z + add.z);
}

template <class TYPE>
vector3<TYPE> vector3<TYPE>::operator- (const vector3<TYPE> &sub) const{
	return vector3<TYPE>(x - sub.x, y - sub.y, z - sub.z);
}

template <class TYPE>
vector3<TYPE> vector3<TYPE>::operator* (TYPE mul) const{
	return vector3<TYPE>(x * mul, y * mul, z * mul);
}

template <class TYPE>
vector3<TYPE> & vector3<TYPE>::operator+= (const vector3<TYPE> &add){
	return (*this = *this + add);
}

template <class TYPE>
vector3<TYPE> & vector3<TYPE>::operator-= (const vector3<TYPE> &sub){
	return (*this = *this - sub);
}

template <class TYPE>
vector3<TYPE> & vector3<TYPE>::operator*= (TYPE mul){
	return (*this = *this * mul);
}

template <class TYPE>
TYPE vector3<TYPE>::dot_product(const vector3<TYPE> &src) const{
	return x*src.get_x() + y*src.get_y() + z*src.get_z();
}

template <class TYPE>
vector3<TYPE> vector3<TYPE>::cross_product(const vector3<TYPE> &src) const{
	vector3<TYPE> cross_vector;
	cross_vector.set(
		y*src.get_z() - z*src.get_y(),
		z*src.get_x() - x*src.get_z(),
		x*src.get_y() - y*src.get_x()
		);

	return cross_vector;
}

template <class TYPE>
vector3<TYPE> & vector3<TYPE>::normalize(void){
	double r = get_r();
	if(r == 0) return *this;

	x = x / r;
	y = y / r;
	z = z / r;

	return *this;
}

/* Access methods *****************************************/
template <class TYPE>
TYPE vector3<TYPE>::get_x(void) const{
	return x;
}

template <class TYPE>
TYPE vector3<TYPE>::get_y(void) const{
	return y;
}

template <class TYPE>
TYPE vector3<TYPE>::get_z(void) const{
	return z;
}

/* Spherical Polar Coordinates ****************************/
/* FIXME 
	TYPE vector3<TYPE>::get_r(void) const{
	constってなんだっけ?
*/
template <class TYPE>
TYPE vector3<TYPE>::get_r(void){
	return sqrt(x*x + y*y + z*z);
}

template <class TYPE>
TYPE vector3<TYPE>::get_theta(void){
	return atan2(sqrt((x*x + y*y)), z);
}

template <class TYPE>
TYPE vector3<TYPE>::get_phi(void){
	return atan2(y, x);
}

/*
	x = r * sin(theta) * cos(phi);
	y = r * sin(theta) * sin(phi);
	z = r * cos(theta);
*/
template <class TYPE>
void vector3<TYPE>::set_polar_coordinate(double r, double theta, double phi){
	x = r * sin(theta);
	y = x * sin(phi);
	x = x * cos(phi);
	z = r * cos(theta);
}

template <class TYPE>
void vector3<TYPE>::transform(double d_r, double d_theta, double d_phi){
	set_polar_coordinate(get_r() + d_r, get_theta() + d_theta, get_phi() + d_phi);
}


/* for Debug **********************************************/
template <class TYPE>
void vector3<TYPE>::output(void){
	const char tab[] = "\t";
	std::cout << x << tab << y << tab << z << std::endl;
}

#endif
