#ifndef __window_sdl
#define __window_sdl

#include <SDL.h>

/**********************************************************
	OpenGLを使用することを目的としたwindowを作成する
	SDLにより実装されている
**********************************************************/
// FLAGの設定
//int init(int width, int height, int pos_x, int pos_y, int bpp, int flags);
// のint flagは以下の定数の組み合わせ
const int WINDOW_RESIZABLE = 1;
const int WINDOW_FULLSCREEN = 2;
const int WINDOW_NOFRAME = 4;
const int WINDOW_DOUBLE_BUFFER = 8;

class gl_window{
public:
	gl_window(void);
	~gl_window(void);

// 作成

	//windowの大きさ等を指定する
	// 0を指定するとディフォルトの値が指定される
	// 成功 1; 失敗 0
	int create(int width, int height, int pos_x, int pos_y, int bpp, int flags);

// 設定
	int resize(int width, int height);
	int set_gl_atrribute(void);

	// windowのOpenGL属性を指定する
	//SDL_GL_DOUBLEBUFFER は init()で設定する
	// 指定しなくても自動的に設定された
	// bpp = 16
	//SDL_GL_RED_SIZE 8
	//SDL_GL_GREEN_SIZE       8
	//SDL_GL_BLUE_SIZE        8
	//SDL_GL_ALPHA_SIZE       8
	//SDL_GL_BUFFER_SIZE      32
	//SDL_GL_DEPTH_SIZE       24
	//SDL_GL_STENCIL_SIZE     8
	//SDL_GL_ACCUM_RED_SIZE   0
	//SDL_GL_ACCUM_GREEN_SIZE 0
	//SDL_GL_ACCUM_BLUE_SIZE  0
	//SDL_GL_ACCUM_ALPHA_SIZE 0
	//int set_gl_atrribute(int red_size, int green_size, int blue_size, int alpah_size,
	//		int buffer_size, int depth_size, int stencil_size,
	//		int accum_red_size, int accum_green_size, int accum_blue_size, int accum_alpah_size);
	//FIXME 属性が指定されたかチェックする関数
	//int check_gl_atrribute(void);

	// window managerの設定
	// FIXME windowにタイトルと付けれるようにする
	//SDL_WM_SetCaption --  ウィンドウのタイトルとアイコン名をセットします。
	//SDL_WM_GetCaption --  ウィンドウのタイトルとアイコン名を取得します
	//SDL_WM_SetIcon --  表示ウィンドウにアイコンをセットします。
	//SDL_WM_IconifyWindow --  ウィンドウをアイコン化/最小化します。
	//SDL_WM_ToggleFullScreen --  フルスクリーンモードを切り換えます。
	//SDL_WM_GrabInput --  マウスとキーボード入力をつかんで離さないようにます。
	// int init_wm();


// 画面の更新
	// 返り値 成功 1, 失敗 0
	// FIXME function name;
	int swap_buffer(void);

// access method
	inline unsigned int get_width(void){return width;};
	inline unsigned int get_height(void){return height;};
	inline unsigned int get_bpp(void){return bpp;};

	// position 使えたり、使えなかったりする
	inline bool is_position(void){return position;};
	inline unsigned int get_pos_x(void){return pos_x;};
	inline unsigned int get_pos_y(void){return pos_y;};

// debug
	void output(void);

private:
	int width;
	int height;
	int bpp;

	bool position;
	int pos_x;
	int pos_y;

	// SDL
	unsigned int SDL_FLAGS;
	const SDL_VideoInfo *info;
	SDL_Surface *surface;

	// OpenGL ATTRIBUTE
	int red_size;	//フレームバッファの赤チャンネルのサイズ(ビット数)
	int green_size;	//フレームバッファの緑チャンネルのサイズ(ビット数)
	int blue_size;	//フレームバッファの青チャンネルのサイズ(ビット数)
	int alpha_size;	//フレームバッファのαチャンネルのサイズ(ビット数)
	int double_buffer;	//0 または 1 の値を取り、ダブルバッファを有効または無効にする。
	int buffer_size;	//フレームバッファのサイズ(ビット数)
	int depth_size;	//デプスバッファのサイズ(ビット数)
	int stencil_size;	//ステンシルバッファのサイズ(ビット数)
	int accum_red_size;	//アキュムレーションバッファの赤チャンネルのサイズ(ビット数)
	int accum_green_size;	//アキュムレーションバッファの緑チャンネルのサイズ(ビット数)
	int accum_blue_size;	//アキュムレーションバッファの青チャンネルのサイズ(ビット数)
	int accum_alpha_size;	//アキュムレーションバッファのαチャンネルのサイズ(ビット数)
};
#endif // __window_sdl
