// STD
#include <iostream>
#include <vector>
using namespace std;

// LIBS
#include <SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>

// lib/hedder
#include "frame_controll.h"
#include "window_sdl.h"

#include "image.h"

#include "gl_texture.h"

#include "font_bdf.h"
#include "draw_bdf.h"

// lib/template
#include "window_template.h"
#include "tetris_template.h"
#include "opengl_template.h"
#include "camera_template.h"
#include "texture_template.h"

const char tab[] = "\t";

/*---------------------------------------------------------------------------
	main_template
---------------------------------------------------------------------------*/
class main_template : public window_template{
public:
	main_template();
	~main_template();
	
	int initialize(void);
	int finalize(void);

	int display(void);
	int resize(int resize_w, int resize_h);
	int event(SDL_Event *not_use);
	int loop(void);
	int keyhold(Uint8* not_use);

	vector<window_template*> task;

	// bdf font
	font_bdf *kanji;
	image* str_image_tetris;
	GLuint texture;
	int initialize_bdf_font(void);
	int display_test_string(void);
	int finalize_bdf_font(void);
private:
};

main_template::main_template(){
	initialize();
}

main_template::~main_template(){
	finalize();
}

int main_template::initialize(void){
	window = new gl_window;
	// FIXME 定数分離
	window->create(400, 400, 100, 100, 0, WINDOW_DOUBLE_BUFFER | WINDOW_RESIZABLE);

	fc = new frame_controll;
	
	window_template *add;
	task.push_back(add = new opengl_template);
	task.push_back(add = new camera_template);
	task.push_back(add = new texture_template);
	task.push_back(add = new tetris_template);

	for(unsigned int i = 0; i < task.size(); i++){
		task[i]->window = window;
		task[i]->fc = fc;
		task[i]->initialize();
	}

	return 0;
}

int main_template::finalize(void){
	for(unsigned int i = 0; i < task.size(); i++){
		task[i]->finalize();
	}
	task.clear();

	if(window){delete window;window = 0;};
	if(fc){delete fc;fc = 0;};
	SDL_Quit();
	return 0;
}

int main_template::resize(int resize_w, int resize_h){
	window->resize(resize_w, resize_h);
	for(unsigned int i = 0; i < task.size(); i++){
		task[i]->resize(resize_w, resize_h);
	}
	return 0;
}

int main_template::display(void){
	for(unsigned int i = 0; i < task.size(); i++){
		task[i]->display();
	}
	window->swap_buffer();
	return 1;
}

// FIXME FPSを描画用、イベント用にわける
int main_template::loop(void){
	if(fc->reset(30) == 0) cout << "error Timer start up.\n";
	while(1){
		fc->next();
		if(event(0)) break;
		keyhold(0);
		display();
		for(unsigned int i = 0; i < task.size(); i++){
			task[i]->loop();
		}
		if(fc->gap() > 0) SDL_Delay(fc->gap());
	}
	return 0;
}

int main_template::event(SDL_Event *not_use){
	SDL_Event event;
	while(SDL_PollEvent(&event) != 0){
		for(unsigned int i = 0; i < task.size(); i++){
			task[i]->event(&event);
		}
		switch(event.type){
			case SDL_QUIT:
				return 1;
				break;
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym){ 
					case SDLK_ESCAPE:
						return 1;
						break;
					default:
						break;
				}
				break;
			case SDL_ACTIVEEVENT: // アプリケーションの可視性に関するイベント構造体
				break;
			case SDL_VIDEORESIZE: // ウィンドウのリサイズイベント構造体
				resize(event.resize.w, event.resize.h);
				break;
			case SDL_VIDEOEXPOSE: // 再描画に関するイベント
				break;
			default:
				break;
		}
	}
	return 0;
}

int main_template::keyhold(Uint8* not_use){
	Uint8 *keystate = SDL_GetKeyState(NULL);
	for(unsigned int i = 0; i < task.size(); i++){
		task[i]->keyhold(keystate);
	}
	return 0;
}

/*---------------------------------------------------------------------------*/
// font bdf
/*---------------------------------------------------------------------------*/
int main_template::initialize_bdf_font(void){
	font_bdf *kanji = new font_bdf;
	if(kanji->open("../resource/mplus_j12r.bdf")){
		cout << "error:font bdf" << endl;
	}
	str_image_tetris = draw_bdf("tetris", kanji);
	make_texture(&texture, str_image_tetris);	// FIXME w, hの2^n制限の回避
	return 0;
}

int main_template::display_test_string(void){
	
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glBindTexture(GL_TEXTURE_2D, texture);

	glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0);	glVertex3f(0.0, 0.0, 0.0);
		glTexCoord2f(0.0, 1.0);	glVertex3f(0.0, 1.0, 0.0);
		glTexCoord2f(1.0, 1.0);	glVertex3f(1.0, 1.0, 0.0);
		glTexCoord2f(1.0, 0.0);	glVertex3f(1.0, 0.0, 0.0);
	glEnd();

	glDisable(GL_TEXTURE_2D);

	return 0;
}

int main_template::finalize_bdf_font(void){
	kanji->close();
	delete kanji;
	delete str_image_tetris;
	return 0;
}

/*---------------------------------------------------------------------------*/
//		int main(int argc, char* argv[]);
/*---------------------------------------------------------------------------*/
int main(int argc, char **argv){
	main_template hoge;
	hoge.loop();
	return 0;
}

