
#include "SDL_create_surface.h"
#include "image.h"

#include <SDL.h>
#include <cstdlib>

void set_palette(SDL_Surface *s, image *p);
SDL_Surface* create_sdl_surface(image *p);

SDL_Surface* create_sdl_surface(image *p){

	/*
	SDL_Surface *SDL_CreateRGBSurfaceFrom(
	void *pixels, int width, int height, int depth[bit], int pitch, 
	Uint32 Rmask, Uint32 Gmask, Uint32 Bmask, Uint32 Amask);
	*/
	SDL_Surface *surface;
	switch(p->color){
		case image::RGBA :
				surface = SDL_CreateRGBSurfaceFrom(
				p->pixels, p->width, p->height, p->bit_per_pixel, p->width * p->bit_per_pixel / 8,
				p->space.rgba.Rmask,	// R
				p->space.rgba.Gmask,	// G
				p->space.rgba.Bmask,	// B
				p->space.rgba.Amask);	// alpha
			break;
		case image::PLTE :
				surface = SDL_CreateRGBSurfaceFrom(
				p->pixels, p->width, p->height, p->bit_per_pixel, p->width * p->bit_per_pixel / 8,
				0, 0, 0, 0);
			set_palette(surface, p);
			break;
		default:
			return 0;
	}

	/* trans */
	switch(p->color){
		case image::RGBA :
			for(int i = 0; i < p->space.rgba.num_trans; i++){
				SDL_SetColorKey(surface, SDL_SRCCOLORKEY | SDL_RLEACCEL, p->space.rgba.trns[i]);
			}
			break;
		case image::PLTE :
			for(int i = 0; i < p->space.plte.num_trans; i++){
				unsigned long int plte_trns =
					(p->space.plte.trns)[i].r << 2 +
					(p->space.plte.trns)[i].g << 2 + 
					(p->space.plte.trns)[i].b;
				SDL_SetColorKey(surface, SDL_SRCCOLORKEY | SDL_RLEACCEL,
					plte_trns
				);
			}
			break;
	}

	return surface;
}

void set_palette(SDL_Surface *s, image *p){
	SDL_Color colors[256];
	for(int i = 0; i < p->space.plte.num_palette; i++){
	  colors[i].r = p->space.plte.palette[i].r;
	  colors[i].g = p->space.plte.palette[i].g;
	  colors[i].b = p->space.plte.palette[i].b;
	}

	// int SDL_SetPalette(SDL_Surface *surface, int flags, SDL_Color *colors, int firstcolor, int ncolors);
	SDL_SetPalette(s, SDL_LOGPAL|SDL_PHYSPAL, colors, 0, p->space.plte.num_palette);

	return;
}

