/*
	render_collada_basic

	TRIANGLESのみで構成されたCOLLADA daeファイルをレンダリングします
	最適化を行わず、COLLADAのスキーマ構造にしたがって、処理しているため、
	実験には有用ですが、実用性はありません。

*/

/* FIXME
	class化
	頂点バッファの使用
*/

// COLLADA
#include <dae.h>
#include <dom.h>
#include <dom/domCOLLADA.h>

#include <GL/gl.h>
#include <GL/glu.h>

#include <stdio.h>
#include <string.h>

#include <iostream>
using namespace std;

int render_dae(DAE *dae);
unsigned int get_max_offset(domInputLocalOffset_Array &input_array);

// gl render
int gl_render(
	GLenum mode,
	domListOfFloats *position_float_array,
	domListOfFloats *normal_float_array,
	domListOfFloats *uv_float_array,
	domListOfUInts *p_array,
	domInputLocalOffset_Array *input_array
	){

	// <input>
	// オフセットの取得
	unsigned int max_offset = 1 + (unsigned int)get_max_offset(*input_array);
	int vertex_offset = 0;
	int normal_offset = -1;
	int texcoord_offset = -1;
	for(int i = 0; i < input_array->getCount(); i++){
		if(strcmp((*input_array)[i]->getSemantic(), "VERTEX") == 0){
			vertex_offset = (*input_array)[i]->getOffset();
		}else if(strcmp((*input_array)[i]->getSemantic(), "NORMAL") == 0){
			normal_offset = (*input_array)[i]->getOffset();
		}else if(strcmp((*input_array)[i]->getSemantic(), "TEXCOORD") == 0){
			texcoord_offset = (*input_array)[i]->getOffset();
		}else{
			// FIXME 他のsemantic
		}
	}

	glBegin(mode);
		int index; // <p> primitive
		int num_of_p = (int)(p_array->getCount());
		for(int current_p = 0; current_p < num_of_p; current_p += max_offset){
			// <input> semantic : vertex
			index = p_array->get(current_p + vertex_offset) * 3;
			glVertex3f(
				position_float_array->get(index),
				position_float_array->get(index + 1),
				position_float_array->get(index + 2)
			);
			// <input> semantic : normal 
			if(normal_offset != -1){
			}
			// <input> semantic : texcoord 
			if(texcoord_offset != -1){
			}
			if(normal_float_array != 0){
				index = p_array->get(current_p + vertex_offset) * 3;
				glNormal3f(
					normal_float_array->get(index),
					normal_float_array->get(index + 1),
					normal_float_array->get(index + 2)
				);
			}
			if(uv_float_array != 0){
			}
		}
	glEnd();

	return 0;
}



int render_dae(DAE *dae){
// <geometry>
	// Geometry数のカウント
	// Geometry数とは，<library_geometry>内の<geometry>の数です．
	// COLLADAドキュメントの中のGeometry要素は，必ずしも１つとは限らないので,
	// 最初に数を取得します．
	int geometry_element_count = (int)(dae->getDatabase()->getElementCount(NULL, "geometry", NULL));
	for(int current_geometry = 0; current_geometry < geometry_element_count; current_geometry++){
		// 現在のGeometryの取得
		domGeometry *this_geometry;
		dae->getDatabase()->getElement((daeElement**)&this_geometry, current_geometry, NULL, "geometry");

		// <geometry><mesh>
		// メッシュの取得
		// Mesh要素は，COLLADAドキュメント<mesh></mesh>で記載されてる部分ですね．
		// ここには，頂点座標や法線，テクスチャ座標など各頂点が持つべきデータや
		// インデックスリストなどを持ちます．
		domMesh *this_mesh = this_geometry->getMesh();


		// <geometry><mesh><source>
		// <geometry><mesh><source><float_array>
		// FIXME <mesh><source> Position, Normal, UVの順番は確定?
		domListOfFloats *position_float_array = 0;
		domListOfFloats *normal_float_array = 0;
		domListOfFloats *uv_float_array = 0;

		int num_of_source = this_mesh->getSource_array().getCount();
		position_float_array = &(this_mesh->getSource_array()[0]->getFloat_array()->getValue());
		if(num_of_source >= 2)
			normal_float_array = &(this_mesh->getSource_array()[1]->getFloat_array()->getValue());
		if(num_of_source == 3)
			uv_float_array = &(this_mesh->getSource_array()[2]->getFloat_array()->getValue());

		// <geometry><mesh><vertices>

		// プリミティブ <geometry><mesh><...>
		/* 各COLLADA プリミティブの子要素
		<lines> <linestrips> <triangles> <trifans> <tristrips>
			<input><p><extra>

		<polygons>
			<input><p><ph><extra>

		<polylist>
			<input><vcount><p><extra> */
		GLenum mode;
		domInputLocalOffset_Array *input_array;
		domListOfUInts *p_array;

		int triangle_element_count = (int)(this_mesh->getTriangles_array().getCount());
		for(int current_triangle = 0; current_triangle < triangle_element_count; current_triangle++){
			mode = GL_TRIANGLES;
			input_array = &(this_mesh->getTriangles_array()[current_triangle]->getInput_array()); // <input>
			p_array = &(this_mesh->getTriangles_array()[current_triangle]->getP()->getValue()); // <p>
		}
		// FIXME 他のプリミティブタイプへの対応

		gl_render(mode,
			position_float_array, normal_float_array, uv_float_array,
			p_array, input_array);

	}

}

unsigned int get_max_offset(domInputLocalOffset_Array &input_array) {
	unsigned int max_offset = 0;
	for (unsigned int i = 0; i < input_array.getCount(); i++) {
		if ( input_array[i]->getOffset() > max_offset) {
			max_offset = (unsigned int)input_array[i]->getOffset();
		}
	}
	return max_offset;
}

