
#include <stack>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "delaunay.h"

delaunay::delaunay()
{
	triangles = new std::vector<triangle>;
}

delaunay::~delaunay()
{
	m_pTri.clear();
}

std::vector<triangle>* delaunay::make_triangle(std::vector<vector3d> *delaunay_vertics)
{
	vertics = delaunay_vertics;
	m_pTri.clear();
	triangles->clear();

// 最大最小値を取得
	double w_max, w_min;
	double h_max, h_min;
	w_max = (*vertics)[0].x;
	w_min = (*vertics)[0].x;
	h_max = (*vertics)[0].y;
	h_min = (*vertics)[0].y;
	for(unsigned int i = 0; i < vertics->size(); i++){
		if(w_max < ((*vertics)[i].x)) w_max = (*vertics)[i].x;
		if(w_min > ((*vertics)[i].x)) w_min = (*vertics)[i].x;
		if(h_max < ((*vertics)[i].y)) h_max = (*vertics)[i].y;
		if(h_min > ((*vertics)[i].y)) h_min = (*vertics)[i].y;
	}

// 大枠を広げる
	w_max += 0.1;
	w_min -= 0.1;
	h_max += 0.1;
	h_min -= 0.1;
		// FIXME const double d;

// 大枠になる点を追加
	int ip0 = add_vertex(w_min, h_min);
	int ip1 = add_vertex(w_max, h_min);
	int ip2 = add_vertex(w_max, h_max);
	int ip3 = add_vertex(w_min, h_max);

// 大枠になる三角形を追加
	add_triangle(ip0, ip1, ip3);
	add_triangle(ip1, ip2, ip3);

// 各点について逐次追加
	for(unsigned int i = 0; i < vertics->size(); i++){
		AddPtPos(i);
	}

// 大枠になる点を含む三角形を削除
	for(std::vector<triangle>::iterator i = triangles->begin();
		i < triangles->end(); i++){
		for(int j = 0; j < 3; j++){
			int k = i->vertics[j];
			if((k == ip0) || (k == ip1) || (k == ip2) || (k == ip3)){
				triangles->erase(i);
			}
		}
	}

// 大枠になる点を削除
	std::vector<vector3d>::iterator pos = (*vertics).begin();
	vertics->erase(pos + ip0);
	vertics->erase(pos + ip1);
	vertics->erase(pos + ip2);
	vertics->erase(pos + ip3);

	m_pTri.clear();
	return triangles;
}

bool delaunay::AddPtPos(int p)
{
	double x = (*vertics)[p].x;
	double y = (*vertics)[p].y;
	int tri;

	// (x, y)を内部に含む三角形iTriを得る
	int iTri = FindTriContains(x, y);
	if (iTri<0){
		std::cout << "error" << std::endl;
		return false;
	}

	// iTriに隣接する三角形を得る．（なければ-1になる）
	int iatri0 = GetAdjacent(iTri, 0);
	int iatri1 = GetAdjacent(iTri, 1);
	int iatri2 = GetAdjacent(iTri, 2);

	// iTriを3つの小三角形に分割したものに置き換える
	int a, b, c;	//iTriの頂点

	a = (*triangles)[iTri].vertics[0];
	b = (*triangles)[iTri].vertics[1];
	c = (*triangles)[iTri].vertics[2];

	adjacent tri0(a, b, p);
	adjacent tri1(b, c, p);
	adjacent tri2(c, a, p);

	m_pTri[iTri] = tri0;

	int itri0 = iTri;
	int itri1 = add_adjacent(tri1);
	int itri2 = add_adjacent(tri2);
	SetAdjacent(itri0, itri1);
	SetAdjacent(itri1, itri2);
	SetAdjacent(itri2, itri0);
	if (iatri2!=-1) SetAdjacent(itri0, iatri2);
	if (iatri0!=-1) SetAdjacent(itri1, iatri0);
	if (iatri1!=-1) SetAdjacent(itri2, iatri1);

	// 3つの小三角形をスタックに積む
	std::stack<int> stack;
	stack.push(itri0);
	stack.push(itri1);
	stack.push(itri2);

	// スタックに三角形がある間繰り返し
	while(!stack.empty()){
		tri = stack.top();
		stack.pop();

		// triに隣接する三角形について繰り返し
		for (int i=0; i<3; i++) {
			int adtri = GetAdjacent(tri, i);
			if (adtri==-1) continue;

			// その隣接三角形のうち、3つの頂点がどれもpではないものを選ぶ
			int ip0, ip1, ip2;
			GetPtOfTri(adtri, ip0, ip1, ip2);
			if ((ip0!=p) && (ip1!=p) && (ip2!=p)) {

				// p(x, y)が3角形の外接円内にあるかどうかチェック．
				double x0, y0, x1, y1, x2, y2;
				double cx, cy, d, r2;

				GetPtPos(ip0, x0, y0);
				GetPtPos(ip1, x1, y1);
				GetPtPos(ip2, x2, y2);

				CalcCircumCircle(x0, y0, x1, y1, x2, y2, cx, cy, r2);
				d = (x-cx)*(x-cx)+(y-cy)*(y-cy);

				if (d<r2) {
					// pは外接円の内部にある．
					// triとadtriの対角線を入れ替え、
					SwapDiagonal(tri, adtri);

					// さらにその2つを新たにスタックに積む
					stack.push(tri);
					stack.push(adtri);
				}
			}
		}
	}
	return true;
}

// vertex *************************************************
int delaunay::add_vertex(double x, double y){
	vertics->push_back(vector3d(x, y, 0.0));
	return vertics->size() - 1;
}

bool delaunay::GetPtPos(int iPt, double& x, double& y)
{
	if ((iPt<0) || ((unsigned int)iPt>=vertics->size())) {
		return false;
	}

	// FIXME
	x = (*vertics)[iPt].x;
	y = (*vertics)[iPt].y;

	return true;
}

// triangles ***********************************************

int delaunay::add_triangle(int a, int b, int c){
	triangles->push_back(triangle(a, b, c));
	return triangles->size() -1;
}


bool delaunay::GetPtOfTri(int iTri, int& ip0, int& ip1, int& ip2)
{
	if ((iTri<0) || ((unsigned int)iTri>=m_pTri.size())) {
		//reporttolog("delaunay::GetPtOfTri(): index(%d) out of range.", iTri);
		return false;
	}

	ip0 = (*triangles)[iTri].vertics[0];
	ip0 = (*triangles)[iTri].vertics[1];
	ip0 = (*triangles)[iTri].vertics[2];

	return true;
}

int delaunay::FindTriContains(double x, double y)
{
	for(unsigned int i=0; i<m_pTri.size(); i++) {
		if (IsInside(i, x, y)) return i;
	}

	return -1;
}

// triangle ***********************************************

bool delaunay::IsInside(int iTri, double x, double y)
{
	if ((iTri<0) || ((unsigned int)iTri>=m_pTri.size())){
		std::cerr << "error : bool delaunay::IsInside(int iTri, double x, double y)"<< std::endl;
		return false;
	}

	int ip0, ip1, ip2;
	double x0, y0, x1, y1, x2, y2;

	GetPtOfTri(iTri, ip0, ip1, ip2);

	GetPtPos(ip0, x0, y0);
	GetPtPos(ip1, x1, y1);
	GetPtPos(ip2, x2, y2);

	return IsInside(x0, y0, x1, y1, x2, y2, x, y);
}

// adjacent ***********************************************
int delaunay::add_adjacent(adjacent pTri)
{
	m_pTri.push_back(pTri);
	return m_pTri.size() - 1;
}

int delaunay::GetAdjacent(int iTri, int iEdge)
{
	if ((iTri<0) || ((unsigned int)iTri>=m_pTri.size())) return -1;
	return m_pTri[iTri].GetAdjacent(iEdge);
}

bool delaunay::IsAdjacent(int iTri0, int iTri1, int& c0, int& c1)
{
	c0 = -1;
	c1 = -1;
	for(int i = 0; i < 3; i++) if(GetAdjacent(iTri0, i) == iTri1) c0 = i;
	for(int i = 0; i < 3; i++) if(GetAdjacent(iTri1, i) == iTri0) c1 = i;
	if(c0 == -1) return false;
	return true;
}

bool delaunay::SetAdjacent(int iTri0, int iTri1)
{
	if ((iTri0==iTri1) || 
	    (iTri0<0) || ((unsigned int)iTri0>=m_pTri.size()) || 
		(iTri1<0) || ((unsigned int)iTri1>=m_pTri.size())) return false;

	// 共通する辺のインデックス
	int edge0, edge1;
	if(!(*triangles)[iTri0].get_common_edge(&((*triangles)[iTri0]), edge0, edge1))
		return false;

	m_pTri[iTri0].SetAdjacent(edge0, iTri1);
	m_pTri[iTri1].SetAdjacent(edge1, iTri0);

	return true;
}

bool delaunay::SwapDiagonal(int iTri0, int iTri1)
{
	if(!(*triangles)[iTri0].swap_diagonal(&((*triangles)[iTri1])))
		return false;

// 隣接三角形の取得
	int adjacent_triangle[4];
	for(int i = 0;0;){
		for(int j = 0; j < 3; j++){
			if(m_pTri[iTri0].GetAdjacent(j) != iTri1){
				adjacent_triangle[i] = m_pTri[iTri0].GetAdjacent(j);
				i++;
			}
		}
		for(int j = 0; j < 3; j++){
			if(m_pTri[iTri1].GetAdjacent(j) != iTri0){
				adjacent_triangle[i] = m_pTri[iTri0].GetAdjacent(j);
				i++;
			}
		}
	}

// 隣接三角形の設定
	for(int i = 0; i < 4; i++) SetAdjacent(iTri0, adjacent_triangle[i]);
	for(int i = 0; i < 4; i++) SetAdjacent(iTri1, adjacent_triangle[i]);
	return true;
}


// 数学 *************************************************
// 三点(a, b, c)で構成される三角形内に(x, y)が含まれるか
bool delaunay::IsInside(double ax, double ay,
						double bx, double by,
						double cx, double cy,
						double x, double y){

	if(IsPlusRotation(bx-ax, by-ay, x-ax, y-ay) == IsPlusRotation(cx-bx, cy-by, x-bx, y-by))
		if(IsPlusRotation(bx-ax, by-ay, x-ax, y-ay) == IsPlusRotation(ax-cx, ay-cy, x-cx, y-cy))
			return true;

	return false;

}

void delaunay::CalcCircumCircle(
	double x0, double y0, double x1, double y1, double x2, double y2,
	double& cx, double& cy, double& r2)
{
	double hx0 = (x0+x1)/2;
	double hy0 = (y0+y1)/2;
	double hx1 = (x1+x2)/2;
	double hy1 = (y1+y2)/2;
	double dx0 = -(y1-y0);
	double dy0 = x1-x0;
	double dx1 = -(y2-y1);
	double dy1 = x2-x1;
	double t, det;
	
	det = dx0*(-dy1)-(-dx1)*dy0;
	if (fabs(det)<0.000001) {
		//reporttolog("Error: deteminant is nearly zero\n");
		//fatal();
	}

	t = ((-dy1)*(hx1-hx0)+dx1*(hy1-hy0))/det;

	// 中心座標	
	cx = hx0+t*dx0;
	cy = hy0+t*dy0;

	// 半径の二乗
	r2 = (x0-cx)*(x0-cx)+(y0-cy)*(y0-cy);
}

bool delaunay::IsPlusRotation(double x0, double y0, double x1, double y1)
{
	return ((x0*y1 - x1*y0) > 0);
}

