
#include "draw_bdf.h"
#include "font_bdf.h"
#include "image.h"

#include <cstdlib>
#include <cstring>
#include <cmath>
#include <iostream>
using namespace std;

unsigned int euc2encoding(const char *kanji);
int init_image(image *img, bdf_glyph **fonts, unsigned int num_fonts);
int get_num_fonts(char *str);
	/* 
	ʸĹ(ѤʸȤ)
	Ⱦѥʤˤ̤б
	*/
void get_fonts(char *str, bdf_glyph** fonts, int num_fonts, font_bdf *kanji);
	/*
	ʸfont_bdfindex¤ӤѴ
	Ⱦѥʤˤ̤б
	return num_glyphs

	eucΤߤб
	ʸɤ˰¸ƤΤϤδؿΤ
	*/
void get_image_size(bdf_glyph **fonts, int num_fonts, image *img);
	/* image->w, image->h  */
int blit_glyphs(bdf_glyph **fonts, int num_fonts, image *img);
void blit_glyph(bdf_glyph *font, image *img, int x0, int y0);


image* draw_bdf(char *str, font_bdf *kanji){
	image *img = new image;
	int num_fonts = get_num_fonts(str);
	bdf_glyph **glyphs
		 = (bdf_glyph **)malloc(num_fonts * sizeof(bdf_glyph*));

	get_fonts(str, glyphs, num_fonts, kanji);
	init_image(img, glyphs, num_fonts);
	blit_glyphs(glyphs, num_fonts, img);
	return img;
}

int get_num_fonts(char *str){
	int len = strlen(str);
	int num_fonts = 0;
	for(int i = 0; i < len; i++){
		num_fonts++;
		if(*(str + i) < 0) i++;	
	}

	return num_fonts;
}


unsigned int euc2encoding(const char *kanji){
	return ((unsigned char)kanji[0] & 0x7f) * 0x100
		 + ((unsigned char)kanji[1] & 0x7f);
}


int init_image(image *img, bdf_glyph **fonts, unsigned int num_fonts){
	get_image_size(fonts, num_fonts, img);

	unsigned int size = img->width / 8 * img->height;
	img->pixels = (unsigned char *)malloc(size * sizeof(unsigned char));
	for(unsigned int i = 0; i < size; i++)
		img->pixels[i] = 0;

	img->bit_per_pixel = 1;
	img->pitch = ((img->width + 7) / 8) * 8;
	img->color = image::RGBA;
	img->space.rgba.Rmask = 0x01;
	img->space.rgba.Gmask = 0x01;
	img->space.rgba.Bmask = 0x01;
	img->space.rgba.Amask = 0x00;
/*
	img->space.rgba.num_trans = 0;
	img->space.rgba.trns = 0;
*/
	img->space.rgba.num_trans = 1;
	img->space.rgba.trns = (unsigned long int*)malloc(img->space.rgba.num_trans * sizeof(unsigned long int));
	img->space.rgba.trns[0] = 0;
	return 0;
}

void get_fonts(char *str, bdf_glyph** fonts, int num_fonts, font_bdf *kanji){
	int len = strlen(str);
	unsigned long int index;
	int j = 0;
	for(int i = 0; i < len; i++){
		if(*(str + i) > 0){
			index = *(str + i);
		}else{
			index = euc2encoding(str + i);
			char high = (index >> 8) & 0xff;
			char low = index & 0xff;
			index = kanji->get_index(high, low);
			i++;
		}

		fonts[j] = kanji->glyph[index];
		j++;
	}
}

void get_image_size(bdf_glyph **fonts, int num_fonts, image *img){
	unsigned int w = 0;

	for(int i = 0; i < num_fonts - 1; i++){
		w += fonts[i]->d;
	}
	w += fonts[num_fonts -1]->w;
	w = ((w + 7) / 8) * 8;


	int h = 0;
	for(int i = 0; i < num_fonts; i++){
		if(h < fonts[i]->h - fonts[i]->y)
			h = fonts[i]->h - fonts[i]->y;
	}


	img->width = w;
	img->height = h;
}


int blit_glyphs(bdf_glyph **fonts, int num_fonts, image *img){

	int x0 = 1;
	int y0 = -2;
	for(int i = 0; i < num_fonts; i++){
		blit_glyph(fonts[i], img, x0, y0);
		x0 += fonts[i]->d;
	}

	return 0;
}

void blit_glyph(bdf_glyph *font, image *img, int x0, int y0){

	/* (x, y) :  */
	int x = x0 + font->x;
	int y = font->h + abs(font->y);

	unsigned char *p_byte = img->pixels + (img->height - y + 1) * img->width  / 8 + (x / 8);
	unsigned int p_bit = x % 8;

	unsigned char *bmp = font->bmp;

	for(int i = 0; i < font->h; i++){
		for(int j = 0; j < ((font->w + 7) / 8); j++){

			*(p_byte + j) |= (*bmp) >> p_bit;
			*(p_byte + j + 1) |= ((*bmp) << (8 - p_bit));
			bmp++;

		}
		p_byte += img->width / 8;
	}
}

bool is_bit_on(unsigned char *pixels, int w, int h, int x, int y){
	return (pixels[((w * y) / 8) + (x / 8)] >> (7 - x % 8)) & 0x01;
}

void expand_bit_image(image *img){
	unsigned char *p;
	p = (unsigned char *)malloc(img->width * img->height * 3);

	for(int y = 0; y < img->height; y++){
		for(int x = 0; x < img->width; x++){
			if(is_bit_on(img->pixels, img->width, img->height, x, y)){
				//cout << "@";
				p[((img->width) * y * 3) + x * 3] = 0xff;
				p[((img->width) * y * 3) + x * 3 + 1] = 0xff;
				p[((img->width) * y * 3) + x * 3 + 2] = 0xff;
			}else{
				//cout << " ";
				p[((img->width) * y * 3) + x * 3] = 0x00;
				p[((img->width) * y * 3) + x * 3 + 1] = 0x00;
				p[((img->width) * y * 3) + x * 3 + 2] = 0x00;
			}
		}
		//cout << endl;
	}

	img->bit_per_pixel = 24;
	img->pitch = (img->width + 7) * (24 / 8);
	img->color = image::RGBA;
	img->space.rgba.Rmask = 0xff;
	img->space.rgba.Gmask = 0xff00;
	img->space.rgba.Bmask = 0xff0000;
	img->space.rgba.Amask = 0x00;

	if(img->pixels) free(img->pixels);
	img->pixels = p;
}


