
#include "font_bdf.h"

#include <cstdlib>
#include <cstdio>
#include <cstring>

#define BUFFER 256 

font_bdf::font_bdf(void){
	for(unsigned int i = 0; i <  9092; i++)
		glyph[i] = 0;
}

font_bdf::~font_bdf(void){
	close();
}

void font_bdf::close(void){
	for(unsigned int i = 0; i <  9092; i++){
		if(glyph[i]){
			free(glyph[i]->bmp);
			free(glyph[i]);
			glyph[i] = 0;
		}
	}
}

int font_bdf::open(char *file_name){

	FILE *fp = fopen(file_name, "r");
	if(fp == 0) return 1;

	long int v[4];

	/* bdf */
	if(get_value(fp, "SIZE", v, 1, 10)) return 2;
	int size = (int)(v[0] & 0xff);

	/* font */
	while(1){
		int g = get_value(fp, "ENCODING", v, 1, 10);
		if(g == 3){
			break;
		}else if(g == 2){
			return 2;
		}else if(g == 0){
			char high = ((v[0] >> 8) & 0xff);
			char low = (v[0] & 0xff);
			int index = get_index(high, low);
			glyph[index] = (bdf_glyph *)malloc(sizeof(bdf_glyph));
			if(!glyph) throw std::bad_alloc();
			bdf_glyph *p_font = glyph[index];

			get_value(fp, "DWIDTH", v, 1, 10);
				p_font->d = v[0];

			get_value(fp, "BBX", v, 4, 10);
				p_font->w = v[0];
				p_font->h = v[1];
				p_font->x = v[2];
				p_font->y = v[3];

			get_value(fp, "BITMAP", v, 0, 16);
			/* 一行[byte] */
			int line_size = (p_font->w + 7) / 8;	/* font->w以上の最小の8の倍数を8で割った商 */
			p_font->bmp = (unsigned char *)malloc(size * line_size * sizeof(unsigned char));
			if(!p_font->bmp) throw std::bad_alloc();

			for(int i = 0; i < size; i++){
				if(get_value(fp, 0, v, 1, 16)) return 2;
				for(int j = 0; j < line_size; j++){
					int rshift = line_size - j - 1;
					p_font->bmp[i * line_size + j] = (v[0] >> (rshift * 8)) & 0xff;
				}
			}
		}
	}

	return 2;
}

int font_bdf::get_index(char high, char low){
	if(high == 0) return (unsigned char)low;
	return (0xff
			+ (high - 0x21) * (0x7e - 0x21)
			+ (low - 0x21));
}

int font_bdf::get_value(FILE *fp, const char *str, long int *v, int num_v, int radix){
	char buf[BUFFER];
	char *p;

	if(str == 0){
		if(!fgets(buf, BUFFER, fp)) return 2;
		v[0] = strtol(buf, 0, radix);
		p = buf;
		for(int i = 1; i < num_v; i++){
			if((p = strchr(buf, ' ')) == 0)	return 2;
			v[i] = strtol(p, 0, radix);
			p++;
		}
		return 0;
	}else{
		while(fgets(buf, BUFFER, fp)){
			if(!strncmp(buf, str, strlen(str))){
				p = buf;
				for(int i = 0; i < num_v; i++){
					if((p = strchr(p, ' ')) == 0)	return 2;
					v[i] = strtol(p, 0, radix);
					p++;
				}
				return 0;
			}
		}
	}

	return 3;
}

