
#include "image_png.h"

#include <cstdlib>
#include <png.h>

using namespace std;

image_png::image_png(void){
}

image_png::~image_png(void){
}

bool image_png::open(char *file_name){

	int color_type;

	FILE *fp = fopen(file_name, "rb");
		if(!fp) return false;
		if(!get_picture_infomation(fp, &color_type)) return false;
		if(!make_pixel_data()) return false;
	fclose(fp);

	/* PLTE */
	if(color_type == 0 | color_type == 3){
			color = PLTE;
			space.plte.palette = 0;
			space.plte.num_palette = 0;
			space.plte.trns = 0;
			space.plte.num_trans = 0;

	}else if( color_type ==  2 | color_type ==  4 | color_type ==  6){
			color = RGBA;
			space.rgba.Rmask= 0;
			space.rgba.Gmask= 0;
			space.rgba.Bmask= 0;
			space.rgba.Amask= 0;
			space.rgba.trns = 0;
			space.rgba.num_trans = 0;
	}else{
		return false;
	}

	get_palette(color_type);

	set_mask(color_type);

	get_trns(color_type);

	return true;
}

bool image_png::get_picture_infomation(FILE *fp, int *color_type){
	// 初期化
	char signature[8];
	fread(signature, 1, 8, fp);
	if(png_sig_cmp((png_bytep)signature, 0, 8)) return false;

	png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
	if (!png_ptr) return false;

	// png_info 構造体の初期化（画像データの前にあるチャンク用）
	info_ptr = png_create_info_struct(png_ptr);
	if (!info_ptr) {
		png_destroy_read_struct(&png_ptr, 0, 0);
		return false;
	}

	png_init_io(png_ptr, fp);
	png_set_sig_bytes(png_ptr, 8);  

	// 画像情報の取得
	png_read_info(png_ptr, info_ptr);

	int interlace_type;
	int compression_type;
	int filter_type;

	// png_get_IHDR(png_struct*, png_info*, png_uint_32*, png_uint_32*, int*,
	// int*, int*, int*, int*);
	png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_per_pixel,
		color_type, &interlace_type, &compression_type, &filter_type);

	switch (*color_type){
		case 0: // グレイスケール
			bit_per_pixel *= 1;
			break;
		case 2: // RGB
			bit_per_pixel *= 3;
			break;
		case 3: // パレット
			bit_per_pixel *= 1;
			break;
		case 4: // グレイスケール+アルファ
			bit_per_pixel *= 2;
			break;
		case 6: // RGB+アルファ
			bit_per_pixel *= 4;
			break;
	}

	return true;
}

bool image_png::make_pixel_data(void){

	pixels = (unsigned char *)malloc(height * width * bit_per_pixel / 8 * sizeof(unsigned char));
	unsigned char **p = (unsigned char **)malloc(height * sizeof(unsigned char *));

	int row_bytes = width * bit_per_pixel / 8 * sizeof(unsigned char);
	p[0] = pixels;
	for(unsigned int i = 1; i < height; i++){
		p[i] = p[i-1] + row_bytes;
	}

	// 読み込み
	png_read_image(png_ptr, (png_byte **)p);

	std::free(p);

	return true;
}

bool image_png::get_palette(int color_type){
	switch (color_type){
		case 0: // グレイスケール
			space.plte.num_palette = 256;
			space.plte.palette = (colors *)malloc(256 * sizeof(colors));
			for(int i = 0; i < 256; i++){
			  space.plte.palette[i].r=i;
			  space.plte.palette[i].g=i;
			  space.plte.palette[i].b=i;
			}
			break;

		case 3: // パレット
			png_colorp png_palette;
			png_get_PLTE(png_ptr, info_ptr, &png_palette, &(space.plte.num_palette));
			space.plte.palette = (colors *)malloc(space.plte.num_palette * sizeof(colors));
			for(int i = 0; i < space.plte.num_palette; i++){
			  space.plte.palette[i].r = png_palette[i].red;
			  space.plte.palette[i].g = png_palette[i].green;
			  space.plte.palette[i].b = png_palette[i].blue;
			}
			break;
	}

	return true;
}

bool image_png::set_mask(int color_type){

	switch (color_type){
		case 2: // RGB
			color = RGBA;
			space.rgba.Rmask = 0x000000ff;
			space.rgba.Gmask = 0x0000ff00;
			space.rgba.Bmask = 0x00ff0000;
			space.rgba.Amask = 0x00000000;
			break;
		case 4: // グレイスケール+アルファ
			color = RGBA;
			space.rgba.Rmask = 0x000000ff;
			space.rgba.Gmask = 0x000000ff;
			space.rgba.Bmask = 0x000000ff;
			space.rgba.Amask = 0x0000ff00;
			break;
		case 6: // RGB+アルファ
			color = RGBA;
			space.rgba.Rmask = 0x000000ff;
			space.rgba.Gmask = 0x0000ff00;
			space.rgba.Bmask = 0x00ff0000;
			space.rgba.Amask = 0xff000000;
			break;
		default:
			return false;
	}

	return true;
}

/*!
tRNSチャンク　　　＜透明度チャン ク＞
png_get_tRNS -- get transparency data for images

png_uint_32 png_get_tRNS(png_structp png_ptr, png_infop info_ptr,
	png_bytep * trans, int * num_trans, png_color_16p * trans_values);

Description
png_get_tRNS() shall obtain the transparency data for paletted images and image types
	that don't need a full alpha channel from info_ptr.
For a paletted image, the function retrieves the transparency values stored
	in the same order as the palette colors, starting from index 0.
For non-paletted images, the function retrieves the single color value
		which is treated as fully transparent.
If the transparency information is valid,
	i.e. PNG_INFO_tRNS bit is set for info_ptr->valid:
	*trans shall be set to the transparency values for a paletted image.
Values for the data shall be in range [0,255],
	ranging from fully transparent to fully opaque, respectively.

*num_trans shall be set to the number of transparency values
*trans_values shall be set to the single color value specified for non-paletted images.

Return Value
Returns PNG_INFO_tRNS on successful return, 0 otherwise. 

typedef struct png_color_16_struct {
    png_byte index;
    png_uint_16 red;
    png_uint_16 green;
    png_uint_16 blue;
    png_uint_16 gray;
} png_color_16;

typedef struct png_color_8_struct {
    png_byte red;
    png_byte green;
    png_byte blue;
    png_byte gray;
    png_byte alpha;
} png_color_8;
*/
bool image_png::get_trns(int color_type){

	png_bytep trans;
	int num_trans = 0;
	png_color_16p trans_values;
	png_get_tRNS(png_ptr, info_ptr, &trans, &num_trans, &trans_values);
	
	if(num_trans)
	switch (color_type){
		case 0: // グレイスケール
			break;
		case 2: // RGB
			break;
		case 3: // パレット

			space.plte.num_trans = num_trans;
			space.plte.trns = (colors *)malloc(num_trans * sizeof(colors));
			for(int i = 0; i < num_trans; i++){
				(space.plte.palette)[i].r = trans_values[i].red;
				(space.plte.palette)[i].g = trans_values[i].green;
				(space.plte.palette)[i].b = trans_values[i].blue;
			}
			break;
	}

	return true;

}

