
#include <iostream>
#include "window_sdl.h"
using namespace std;

gl_window::gl_window(void){

	// window
	width = 0;
	height = 0;
	bpp = 0;

	// position
	position = false; 
	pos_x = 0;
	pos_y = 0;

	// SDL
	SDL_FLAGS = SDL_OPENGL;
	info = 0;
	surface = 0;

	// OpenGL ATTRIBUTE
	red_size = 0;
	green_size = 0;
	blue_size = 0;
	alpha_size = 0;
	double_buffer = 0;
	buffer_size = 0;
	depth_size = 0;
	stencil_size = 0;
	accum_red_size = 0;
	accum_green_size = 0;
	accum_blue_size = 0;
	accum_alpha_size = 0;
}

gl_window::~gl_window(void){
	SDL_FreeSurface(surface);
}

/*	SDL_FLAGS
	強制
	SDL_SWSURFACE	システムメモリからビデオサーフェスを作成します。
	SDL_HWSURFACE	ビデオメモリからビデオサーフェスを作成します。
	SDL_OPENGL	OpenGL の描画コンテキストを作成します。前もって SDL_GL_SetAttribute によって OpenGL のビデオ属性を設定しておく必要があります。

	選択
	SDL_RESIZABLE	リサイズ可能なウィンドウを作成します。ユーザーの手でウィンドウがリサイズされた場合は、 SDL_VIDEORESIZE イベントが発生し、新しいサイズで SDL_SetVideoModeが再度呼ばれることがあります。
	SDL_FULLSCREEN	SDL はフルスクリーンモードの使用を試みます。どういう理由であれ、ハードウェアによる解像度変更ができない場合は一段階解像度の高いモードが使われ、黒い背景の中央に表示ウインドウが置かれます。
	SDL_NOFRAME	もし可能であれば、SDL_NOFRAME は SDL がタイトルバーなし、あるいはフレームによる装飾なしのウィンドウが生成することになります。フルスクリーンモードは自動的にこのフラグを設定します。
*/
int gl_window::create(int w, int h, int x, int y, int b, int flag){

	// 引数の適応
	if(w != 0) width = w;
	if(h != 0) height = h;
	if(b != 0) bpp = b;
	if(x != 0) pos_x = x;
	if(y != 0) pos_y = y;
	if(flag & WINDOW_RESIZABLE) SDL_FLAGS |= SDL_RESIZABLE;
	if(flag & WINDOW_FULLSCREEN) SDL_FLAGS |= SDL_FULLSCREEN;
	if(flag & WINDOW_NOFRAME) SDL_FLAGS |= SDL_NOFRAME;
	if(flag & WINDOW_DOUBLE_BUFFER) double_buffer = 1;

	// SDLサブシステムの初期化
	if(SDL_WasInit(SDL_INIT_VIDEO) == 0)
		if(SDL_InitSubSystem(SDL_INIT_VIDEO) < 0){
			cerr << "gl_window::create()" << endl;
			cerr << "ビデオの初期化に失敗しました" << endl;
			cerr << SDL_GetError() << endl;
			return 0;
	}

	// ビデオ情報を取得
    info = SDL_GetVideoInfo();
	if(info->hw_available){
		SDL_FLAGS |= SDL_HWSURFACE;
	}else{
		SDL_FLAGS |= SDL_SWSURFACE;
	}
	if(bpp == 0) bpp = info->vfmt->BitsPerPixel;

	// OpenGL属性の適応
	set_gl_atrribute();

	// サーフェイスの作成
	surface = SDL_SetVideoMode(width, height, bpp, SDL_FLAGS);
	if(surface == 0){
		cerr << "gl_window::crate()" << endl;
		cerr << "ビデオモードのセットに失敗しました" << endl;
		cerr << SDL_GetError() << endl;
		return 0;	// FIXME 例外処理を実装
	}
	return 1;
}

int gl_window::set_gl_atrribute(void){
	if(red_size) SDL_GL_SetAttribute(SDL_GL_RED_SIZE, red_size);
	if(green_size) SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, green_size);
	if(blue_size) SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, blue_size);
	if(alpha_size) SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, alpha_size);

	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, double_buffer);

	if(depth_size) SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, depth_size);
	if(stencil_size) SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, stencil_size);

	if(accum_red_size) SDL_GL_SetAttribute(SDL_GL_ACCUM_RED_SIZE, accum_red_size);
	if(accum_green_size) SDL_GL_SetAttribute(SDL_GL_ACCUM_GREEN_SIZE, accum_green_size);
	if(accum_blue_size) SDL_GL_SetAttribute(SDL_GL_ACCUM_BLUE_SIZE, accum_blue_size);
	if(accum_alpha_size) SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE, accum_alpha_size);

	return 1;
}

int gl_window::resize(int w, int h){
	width = w;
	height = h;
	SDL_SetVideoMode(width, height, bpp, SDL_FLAGS);
	return 0;
}


int gl_window::swap_buffer(void){
	if(double_buffer == 1) SDL_GL_SwapBuffers();
	else SDL_UpdateRect(surface, 0, 0, 0, 0);
	return 0;
}

void gl_window::output(void){
	const char tab[] = "\t";
	cout << "width" << tab << width << endl;
	cout << "height" << tab << height << endl;
}

/*
int check_gl_atrribute(void){
	if(red_size != SDL_GL_GetAttribute(SDL_GL_RED_SIZE, red_size));
	if(green_size != SDL_GL_GetAttribute(SDL_GL_GREEN_SIZE, green_size));
	if(blue_size != SDL_GL_GetAttribute(SDL_GL_BLUE_SIZE, blue_size));
	if(alpha_size != SDL_GL_GetAttribute(SDL_GL_ALPHA_SIZE, alpha_size));

	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, double_buffer);

	if(depth_size) SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, depth_size);
	if(stencil_size) SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, stencil_size);

	if(accum_red_size) SDL_GL_SetAttribute(SDL_GL_ACCUM_RED_SIZE, accum_red_size);
	if(accum_green_size) SDL_GL_SetAttribute(SDL_GL_ACCUM_GREEN_SIZE, accum_green_size);
	if(accum_blue_size) SDL_GL_SetAttribute(SDL_GL_ACCUM_BLUE_SIZE, accum_blue_size);
	if(accum_alpah_size) SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE, accum_alpha_size);
}
*/
#if 0
int gl_window::set_gl_atrribute(int red, int green, int blue, int alpha,
		int buffer, int depth, int stencil,
		int accum_red, int accum_green, int accum_blue, int accum_alpha){
	// OpenGL ATTRIBUTE
	red_size = red;
	green_size = green;
	blue_size = blue;
	alpha_size = alpha;
	buffer_size = buffer;
	depth_size = depth;
	stencil_size = stencil;
	accum_red_size = accum_red;
	accum_green_size = accum_green;
	accum_blue_size = accum_blue;
	accum_alpha_size = accum_alpha;

	return 1;
}
#endif

