#ifndef __opengl_template
#define __opengl_template

#include <SDL.h>

#include "window_template.h"


/*---------------------------------------------------------------------------*/
/* opengl_template */
/*---------------------------------------------------------------------------*/
class opengl_template : public window_template{
public:
	opengl_template(void);
	~opengl_template(void);

	int initialize(void);
	int finalize(void);

	int display(void);
	int resize(int resize_w, int resize_h);
	int event(SDL_Event *event);
	int keyhold(Uint8* keystate);
	int loop(void);

	int init_lighting(void);
private:
};

opengl_template::opengl_template(void){
}

opengl_template::~opengl_template(void){
}

inline int opengl_template::initialize(void){
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glShadeModel(GL_FLAT);
	glEnable(GL_DEPTH_TEST);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glViewport(0, 0, window->get_width(), window->get_height());
	init_lighting();
	return 1;
}

inline int opengl_template::finalize(void){
	return 0;
}

inline int opengl_template::display(void){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	return 0;
}

inline int opengl_template::resize(int resize_w, int resize_h){
	glViewport(0, 0, (GLsizei)resize_w, (GLsizei)resize_h);
	return 0;
}

inline int opengl_template::event(SDL_Event* event){
	return 0;
}

inline int opengl_template::loop(void){
	return 0;
}

inline int opengl_template::init_lighting(void){
	GLfloat light_position[] = {2.0, -2.0, 2.0, 0.0};
	GLfloat white_light[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat lmodel_ambient[] = {0.3, 0.3, 0.3, 1.0};
	
	GLfloat mat_specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat mat_shininess[] = {50.0};
	
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light);
	glLightfv(GL_LIGHT0, GL_SPECULAR, white_light);

	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	return 0;
}

inline int opengl_template::keyhold(Uint8* keystate){
	return 0;
}

#endif
