
#ifndef __tetris_template
#define __tetris_template

#include <SDL.h>

#include "window_template.h"

#include "tetris.h"
#include "gl_parts.h"
/*---------------------------------------------------------------------------*/
/* tetris_template */
/*	OpenGLを使用。事前に初期化すること */
/*---------------------------------------------------------------------------*/
class tetris_template : public window_template{
public:
	tetris_template(void);
	~tetris_template(void);
	
	int initialize(void);
	int finalize(void);

	int display(void);
	int resize(int resize_w, int resize_h);
	int event(SDL_Event *event);
	int keyhold(Uint8* keystate);
	int loop(void);

private:
	tetris game;
};

tetris_template::tetris_template(void){
}

tetris_template::~tetris_template(void){
}

inline int tetris_template::initialize(void){
	game.init();
	return 0;
}

inline int tetris_template::finalize(void){
	return 0;
}

inline int tetris_template::display(void){
	// マップの描画
	for(int i = -1; i < 11; i++){
		for(int j = 0; j < 19; j++){
			if(game.get_map(i, j)){
				gl_draw_square(200 - i * 20, 360 - j * 20,
					20, 20, window->get_width(), window->get_height());
			}
		}
	}

	// ブロックの描画
	for(int i = 0; i < 5; i++){
		for(int j = 0; j < 5; j++){
			if(game.get_block(i, j)){
				gl_draw_square(200 - (i + game.get_block_x()) * 20,
					360 - (j + game.get_block_y()) * 20,
					20, 20, window->get_width(), window->get_height());
			}
		}
	}

	return 0;
}

inline int tetris_template::resize(int resize_w, int resize_h){
	return 0;
}

inline int tetris_template::event(SDL_Event *event){
	switch(event->type){
		case SDL_KEYDOWN:
			switch(event->key.keysym.sym){
				case SDLK_UP:
					game.turn_right();
					break;
				case SDLK_SPACE:
					game.move_buttom();
					break;
				case SDLK_DOWN:
					game.move_down();
					break;
				case SDLK_LEFT:
					game.move_left();
					break;
				case SDLK_RIGHT:
					game.move_right();
					break;
				default:
					break;
			}
			break;
		default:
			break;
	}
	return 0;
}

inline int tetris_template::loop(void){
 	// 1秒ごとの処理
	if(((fc->frame()) % (fc->base())) == 0){
		if(game.decide()){
			game.move_down();
		}else{
			// game over
			game.clear_map();
		}
	}
	return 0;
}

inline int tetris_template::keyhold(Uint8* keystate){
	return 0;
}

#endif
