
#include "vector3.h"
/* test vector3d vector3<double> */

#include <boost/test/unit_test.hpp>
using namespace boost::unit_test_framework;

#include <iostream>
#include <iomanip>
using namespace std;

void constructors_test(void){
	double src[] = {1.0, 1.0, 1.0};

	vector3d A;
	BOOST_CHECK_EQUAL(A.get_x(), 0.0);
	BOOST_CHECK_EQUAL(A.get_y(), 0.0);
	BOOST_CHECK_EQUAL(A.get_z(), 0.0);

	vector3d B(1.0, 1.0, 1.0);
	BOOST_CHECK_EQUAL(B.get_x(), 1.0);
	BOOST_CHECK_EQUAL(B.get_y(), 1.0);
	BOOST_CHECK_EQUAL(B.get_z(), 1.0);

	vector3d C(src);
	BOOST_CHECK_EQUAL(C.get_x(), 1.0);
	BOOST_CHECK_EQUAL(C.get_y(), 1.0);
	BOOST_CHECK_EQUAL(C.get_z(), 1.0);

	A.set(0.5, 0.5, 0.5);
	BOOST_CHECK_EQUAL(A.get_x(), 0.5);
	BOOST_CHECK_EQUAL(A.get_y(), 0.5);
	BOOST_CHECK_EQUAL(A.get_z(), 0.5);

	A.set(src);
	BOOST_CHECK_EQUAL(A.get_x(), 1.0);
	BOOST_CHECK_EQUAL(A.get_y(), 1.0);
	BOOST_CHECK_EQUAL(A.get_z(), 1.0);
}

void compare_test(void){
	vector3d A(1.0, 1.0, 1.0);
	vector3d B(1.0, 1.0, 1.0);
	vector3d C(0.0, 0.0, 0.0);

	BOOST_CHECK(A == B);
	BOOST_CHECK(B != C);
	BOOST_CHECK(C != A);
}

void calculate_test(void){
	vector3d A;
	vector3d B;
	vector3d C;

	BOOST_CHECK(+(vector3d(1.0, 1.0, 1.0)) == vector3d(1.0, 1.0, 1.0));
	BOOST_CHECK(-(vector3d(1.0, 1.0, 1.0)) == vector3d(-1.0, -1.0, -1.0));
	BOOST_CHECK(vector3d(1.0, 1.0, 1.0) + vector3d(1.0, 1.0, 1.0) == vector3d(2.0, 2.0, 2.0));
	BOOST_CHECK(vector3d(1.0, 1.0, 1.0) - vector3d(1.0, 1.0, 1.0) == vector3d(0.0, 0.0, 0.0));
	BOOST_CHECK(vector3d(1.0, 2.0, 3.0) * 2.0  == vector3d(2.0, 4.0, 6.0));

	A.set(1.0, 1.0, 1.0);
	BOOST_CHECK((A += vector3d(1.0, 1.0, 1.0)) == vector3d(2.0, 2.0, 2.0));
	A.set(1.0, 1.0, 1.0);
	BOOST_CHECK((A -= vector3d(1.0, 1.0, 1.0)) == vector3d(0.0, 0.0, 0.0));
	B.set(1.0, 2.0, 3.0);
	BOOST_CHECK((B *= 2.0) == vector3d(2.0, 4.0, 6.0));

}

void polar_coordinate_test(void){
	/*
	ľɸʤΤǷ׻
	Ƿ̤ͤѤ뤫
	*/

	vector3d A;
	vector3d B;

	A.set(2.0, 3.0, 4.0);
	B.set_polar_coordinate(A.get_r(), A.get_theta(), A.get_phi());
	BOOST_CHECK(A == B);

	double d_r = 2.0;
	double d_theta = 2.0;
	double d_phi = 2.0;
	A.set(2.0, 3.0, 4.0);
	B.set_polar_coordinate(A.get_r() + d_r, A.get_theta() + d_theta, A.get_phi() + d_phi);
	A.transform(d_r, d_theta, d_phi);
	BOOST_CHECK(A == B);


	// r < 0.0 ΤȤ
	A.set(2.0, 3.0, 4.0);
	B.set_polar_coordinate(- A.get_r(), A.get_theta(), A.get_phi());
	A.set(-2.0, -3.0, -4.0);
	BOOST_CHECK(A == B);
	
	// theta, phiϰϳ
	// 8.88178e-16ФΤΩʤ
	/*
	A.set(2.0, 3.0, 4.0);
	B.set_polar_coordinate(A.get_r(), A.get_theta() - 2 * M_PI, A.get_phi() - 2 * M_PI);
	BOOST_CHECK(A == B);
	*/
}

// ѡ
void product_test(void){
	vector3d A(1.0, 2.0, 3.0);
	vector3d B(4.0, 5.0, 6.0);

	BOOST_CHECK(A.dot_product(B) == 32.0);
	BOOST_CHECK(A.cross_product(B) == vector3d(-3.0, 6.0, -3.0));
}

test_suite *init_unit_test_suite(int argc, char *argv[]){
	test_suite *test = BOOST_TEST_SUITE("vector3<double> test");
	test->add(BOOST_TEST_CASE(&constructors_test));
	test->add(BOOST_TEST_CASE(&compare_test));
	test->add(BOOST_TEST_CASE(&calculate_test));
	test->add(BOOST_TEST_CASE(&polar_coordinate_test));
	test->add(BOOST_TEST_CASE(&product_test));

	return test;
}





